<?php
/**
 * @package User Categories
 * @version 0.1.0
 */
/*
Plugin Name: User Categories
Plugin URI: http://www.warna.info/
Description: ユーザーにカスタム分類を適用し、ユーザーのカテゴライズを可能にします。
Author: Hitoshi Omagari
Version: 0.1.1
Author URI: http://www.warna.info/
*/

class User_Categories {
function __construct() {
	add_action( 'init', array( &$this		, 'add_user_category_taxonomy' ) );
	if ( is_admin() ) {
		// ユーザカテゴリメニューの追加
		add_action( 'admin_menu'									, array( &$this, 'add_user_category_menu' ) );

		// ユーザーカテゴリのカラムを修正
		add_filter( 'manage_edit-user_category_columns'				, array( &$this, 'add_user_category_count_columns' ) );
		add_filter( 'manage_user_category_custom_column'			, array( &$this, 'display_user_category_count_custom_column' ), 10, 3 );
		add_filter( 'manage_edit-user_category_sortable_columns'	, array( &$this, 'add_count_sortable_columns' ) );
		add_action( 'admin_print_styles-edit-tags.php'				, array( &$this, 'add_user_category_field_styles' ) );

		// ユーザ一覧にユーザーカテゴリを追加
		add_filter( 'manage_users_columns'							, array( &$this, 'add_user_category_columns' ) );
		add_filter( 'manage_users_custom_column'					, array( &$this, 'display_user_category_custom_column' ), 10, 3 );
//		add_action( 'admin_print_styles-users.php'					, array( &$this, 'add_user_categories_field_styles' ) );

		// ユーザーカテゴリフィールドを追加
		add_action( 'personal_options'								, array( &$this, 'add_user_category_item' ) );
		add_action( 'profile_update'								, array( &$this, 'update_user_category_terms' ), 10, 2 );
		
		// ユーザーカテゴリ用のCSSを出力
		add_action( 'admin_print_styles-profile.php'				, array( &$this, 'add_user_category_list_styles' ) );
		add_action( 'admin_print_styles-user-edit.php'				, array( &$this, 'add_user_category_list_styles' ) );
		
		// ユーザーカテゴリーパラメータの有効化
		add_action( 'pre_user_query'								, array( &$this, 'allow_user_cat_query' ) );
	}
}


function add_user_category_taxonomy() {
	$args = array(
		'label' => 'ユーザーカテゴリー',
		'labels' => array(
			'name' => 'ユーザーカテゴリー',
			'singular_name' => 'ユーザーカテゴリー',
			'search_items' => 'ユーザーカテゴリーを検索',
			'popular_items' => '登録の多いユーザーカテゴリー',
			'all_items' => 'すべてのユーザーカテゴリー',
			'parent_item' => '親ユーザーカテゴリー',
			'edit_item' => 'ユーザーカテゴリーの編集',
			'update_item' => '更新',
			'add_new_item' => 'ユーザーカテゴリーの追加',
			'new_item_name' => '新しいユーザーカテゴリー',
		),
		'public' => false,
		'show_ui' => true,
		'hierarchical' => true,
		'show_tagcloud' => true,
		'show_in_nav_menus' => true
	);
	register_taxonomy( 'user_category', array(), $args);
}


function add_user_category_menu() {
	global $submenu;
	if ( current_user_can( 'edit_user' ) ) {
		$submenu['users.php'][20] = array( 'ユーザーカテゴリー', 'edit_user', 'edit-tags.php?taxonomy=user_category' );
	}
}


function add_user_category_count_columns( $columns ) {
	unset( $columns['posts'] );
	$columns['user'] = 'ユーザー';
	return $columns;
}


function display_user_category_count_custom_column( $output, $column_name, $term_id ) {
	switch ( $column_name ) {
	case 'user' :
		$term = get_term( (int)$term_id, 'user_category' );
		$output = $term->count;
		break;
	default :
	}
	return $output;
}


function add_count_sortable_columns( $sortable ) {
	$sortable['user'] = 'count';
	return $sortable;
}


function add_user_category_columns( $columns ) {
	$new_columns = array();
	foreach ( $columns as $column_name => $column_display_name ) {
		if ( $column_name == 'email' ) {
			$new_columns['user_category'] = 'ユーザーカテゴリー';
		}
		$new_columns[$column_name] = $column_display_name;
	}
	return $new_columns;
}


function display_user_category_custom_column( $output, $column_name, $user_id ) {
	switch ( $column_name ) {
	case 'user_category' :
		$user_cats = $this->get_the_user_categories( $user_id );
		if ( $user_cats ) {
			$cats_arr = array();
			foreach ( $user_cats as $user_cat ) {
//				$link = add_query_arg( array( 'taxonomy' => 'user_category', 'term' => $user_cat->slug ) );
				$cats_arr[] = esc_html( $user_cat->name );
			}
			$output = implode( ', ', $cats_arr );
		}
	break;
	default :
	}
	return $output;
}


function add_user_category_item() {
	global $userdata;
	if ( defined( 'IS_PROFILE_PAGE' ) && IS_PROFILE_PAGE ) {
		$user_id = $userdata->ID;
	} else {
		$user_id = (int)$_GET['user_id'];
	}

	$categories = get_terms( 'user_category', 'orderby=name&hide_empty=0' );
	if ( empty( $categories ) ) { return; }
	$user_cats = $this->get_the_user_categories( $user_id );
	$user_cats = $this->get_opject_ids( $user_cats, 'term_id' );
?>
<tr>
	<th>ユーザーカテゴリー</th>
	<td>
		<input type="hidden" name="tax_input[user_category][]" value="0" />
		<ul id="user_category_checklist" class="list-user_category categorychecklist form-no-clear">
			<?php wp_terms_checklist( $user_id, array( 'taxonomy' => 'user_category', 'checked_ontop' => false ) ) ?>
		</ul>

	</td>
</tr>
<?php
}


function update_user_category_terms( $user_id, $old_user_data  ) {
	if ( isset( $_POST['tax_input']['user_category'] ) && is_array( $_POST['tax_input']['user_category'] ) ) {
		$user_categories = stripslashes_deep( $_POST['tax_input']['user_category'] );
		$user_categories = array_map( 'intval', $user_categories );
		wp_set_object_terms( $user_id, $user_categories, 'user_category' );
	}
}


function allow_user_cat_query( $user_query ) {
	global $wpdb;

	if ( isset( $user_query->query_vars['user_cat_id'] ) ) {
		$params = $user_query->query_vars['user_cat_id'];
		if ( is_string( $params ) ) {
			$ids = explode( ',', $params );
		} elseif ( ! is_array( $params ) ) {
			return;
		}
		$params = array_map( 'intval', $params );
		$field = 'term_id';

	} elseif ( isset( $user_query->query_vars['user_cat'] ) ) {
		$params = $user_query->query_vars['user_cat'];
		if ( is_string( $params ) ) {
			$params = explode( ',', $params );
		} elseif ( ! is_array( $params ) ) {
			return;
		}
		$params = array_map( 'trim', $params );
		$field = 'slug';
	} else {
		return;
	}
	$args = array(
		array(
			'taxonomy' => 'user_category',
			'field' => $field,
			'terms' => $params,
			'operator' => 'IN'
		)
	);
	$tax_query = new WP_Tax_Query( $args );
	$tax_sql = $tax_query->get_sql( $wpdb->users, 'ID' );
	$user_query->query_from .= $tax_sql['join'];
	$user_query->query_where .= $tax_sql['where'] . ' GROUP BY ' . $wpdb->users . '.ID';
}


function get_the_user_categories( $user_id ) {
	return get_the_terms( (int)$user_id, 'user_category' );
}


function add_user_category_list_styles() {
?>
<style type="text/css" charset="utf-8">
#user_category_checklist .children {
	margin-left: 18px;
}
</style>
<?php
}


function add_user_category_field_styles() {
?>
<style type="text/css" charset="utf-8">
.fixed .column-user {
	width: 15%;
	text-align: center;
}
</style>
<?php
}


function add_user_categories_field_styles() {
?>
<style type="text/css" charset="utf-8">
</style>
<?php
}


function get_opject_ids( $objects, $field = 'ID' ) {
	if ( ! is_array( $objects ) ) { return array(); }
	$ids = array();
	foreach( $objects as $object ) {
		if ( isset( $object->$field ) ) {
			$ids[] = $object->$field;
		}
	}
	return $ids;
}


} // class end
$user_categories = new User_Categories;


function get_the_user_categories( $user_id ) {
	global $user_categories;
	return $user_categories->get_the_user_categories( $user_id );
}